%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% This code solves a simple SIR disease model
%% with vital dynamics and plots the results.
%% Written by Calistus N. Ngonghala
%% Dept. of Math. West Virginia University, USA
%% email: cngongha@mix.wvu.edu; ngonghala@yahoo.com
%% June 30, 2010
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function SIRVitalDyn
clear all;
close all;
clc
%%%%%%%%%%%%%%%%%%% Parameter Values %%%%%%%%%%%%%%%%%%%%
prompt={'Enter the contact rate, \beta: ',...
        'Enter birth/death rate, \mu: ',...
        'Enter the removal rate, \nu: ',...
        'Enter the population size, N: ',...
        'Enter the initial time, t_1: ',...
        'Enter the final time, t_f: ',...
        'Enter the initial number of susceptibles individuals, S_1: ',...
        'Enter the initial number of infectious individuals, I_1: ',...
        'Enter the initial number of Recovered individuals, R_1: '};
name='Input for SIR Model with Vital Dynamics';
numlines=1;
defaultanswer={'0.003','0.02','1','1000','0','1000','999','1','0'};
options.Resize='on';
options.WindowStyle='normal';
options.Interpreter='tex';
parameters=inputdlg(prompt,name,numlines,defaultanswer,options);
%%%%%%%%%%%%%%%%%%%%%% Convert cell arrays to numbers %%%%%%%%%%%%%%%%%%%%%
beta = str2num(parameters{1});
mu   = str2num(parameters{2});
nu   = str2num(parameters{3});
N    = str2num(parameters{4});
t_1  = str2num(parameters{5});
t_f  = str2num(parameters{6});
S_1  = str2num(parameters{7});
I_1  = str2num(parameters{8});
R_1  = str2num(parameters{9});

%%%%%%%%%%%%%%%%% Solve system %%%%%%%%%%%%%%%%%%%%%%%%%
opt = odeset('AbsTol',1e-6,'RelTol',1e-5);
[t x] = ode45(@SIRVital,[t_1 t_f],[S_1 I_1 R_1],opt);
R_0 = beta*N/(mu + nu) % Calculate R_0
%%%%%%%%%%%%%%%%%%%%% Plot Results %%%%%%%%%%%%%%%%%%%%%
figure(1)
plot(t,x(:,1),'g',t,x(:,2),'r',t,x(:,3),'b','LineWidth',2)
title('SIR DISEASE MODEL')
xlabel('time')
ylabel('S, I, R');
legend('S', 'I', 'R');
pause(1);
figure(2)
plot(x(:,1),x(:,2),'LineWidth',2)
xlabel('Susceptible Individuals')
ylabel('Infectious Individuals');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% Nested Function %%%%%%%%%%%%%%%%%%
%%%%%%%% This function contains the derivatives %%%%%%%%
function xdot = SIRVital(t,x)
S = x(1);
I = x(2);
R = x(3);
xdot(1) = mu*N - beta*S*I - mu*S;
xdot(2) = beta*S*I - (mu + nu)*I;
xdot(3) = nu*I - mu*R;
xdot = [xdot(1);xdot(2);xdot(3)];
end
end