
function SEIRSModel(~)
clear all
clc

set(0,'defaultaxesfontsize',30,'defaulttextinterpreter','latex');

beta = 0.9;
delta = 0.01;
gamma = 0.2;
mu = 0.01;
sigma = 0.1;
nu = 0.001;

S0 = 999;
E0 = 1;
I0 = 0;
R0 = 0;

x0 = [S0 E0 I0 R0];

Timepsan = [0 300];

[t,x] = ode45(@SEIRS,Timepsan,x0);

S = x(:,1);
E = x(:,2);
I = x(:,3);
R = x(:,4);
N = S + E + I + R;

plot(t,S,'b',t,E,'m',t,I,'r',t,R,'g',t,N,'k','LineWidth',4)
xlabel('Time in days')
ylabel('Populations')
g = legend('Susceptible, $S$','Exposed, $E$','Infectious, $I$','Recovered, $R$','Total population, $N$')
set(g,'Interpreter','latex');
    function xdot = SEIRS(~,x)
        S = x(1);
        E = x(2);
        I = x(3);
        R = x(4);
        
        N = S+E+I+R;
        
        Lambda = mu*N;
        
        Sdot = Lambda + delta*R - ((beta*I/N) + mu)*S;
        Edot = (beta*I/N)*S - (mu + sigma)*E;
        Idot = sigma*E - (mu + nu + gamma)*I;
        Rdot = gamma*I - (mu + delta)*R;
        xdot = [Sdot;Edot;Idot;Rdot];
    end
end