%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% This code solves the system of equations from the deterministic SEIRS
%% model develpoed by Ngwa and Shu and plots the results.
%% Written by Calistus N. Ngonghala
%% Dept. of Math. West Virginia University, USA
%% email: cngongha@mix.wvu.edu; ngonghala@yahoo.com
%% July 1, 2010
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ODESEIRS
clear all;
close all;
clc
%%%%%%%%%%%%%%%%%% Request parameter values from user %%%%%%%%%%%%%%%%%%%%%
  prompt={'Enter the value of \gamma ',...
          'Enter the value of \xi ',...
          'Enter the value of \alpha ',...
          'Enter the value of r',...
          'Enter the value of \beta',...
          'Enter the value of \lambda',...
          'Enter the value of \nu',...
          'Enter the value of a',...
          'Enter the value of b',...
          'Enter the value of c',...
          'Enter the value of e'};
name='Parameter Values for SEIRS Malaria Model';
numlines=1;
defaultanswer={'0.001','4.066','0.3','0.2','0.35','0.00184','2.0','1.002','10.0','1.0','2.4'};
options.Resize='on';
options.WindowStyle='normal';
options.Interpreter='tex';
parameters=inputdlg(prompt,name,numlines,defaultanswer,options);
%%%%%%%%%%%%%%%%%%% Request Initial data from user %%%%%%%%%%%%%%%%%
 prompt1={'Enter the starting time, t_1',...
          'Enter the final time t_f',...
          'Enter the initial proportion of susceptible humans',...
          'Enter the initial proportion of infectious humans',...
          'Enter the initial proportion of immune humans',...
          'Enter the initial proportion of susceptible vectors',...
          'Enter the initial proportion of infectious vectors'};
name1='Initial data';
numlines=1;
defaultanswer1={'0','100','1','0','0','0.9','0.1'};
options1.Resize='on';
options1.WindowStyle='normal';
options1.Interpreter='tex';
initialDat=inputdlg(prompt1,name1,numlines,defaultanswer1,options1);      
%%%%%%%%%%%%%%%%%%%%%% Convert cell arrays to numbers %%%%%%%%%%%%%%%%%%%%%
gamma  = str2num(parameters{1}); 
xi     = str2num(parameters{2});
alpha  = str2num(parameters{3});
r      = str2num(parameters{4});
beta   = str2num(parameters{5});
lambda = str2num(parameters{6});
nu     = str2num(parameters{7});
a      = str2num(parameters{8});
b      = str2num(parameters{9});
c      = str2num(parameters{10});
e      = str2num(parameters{11});
t_1    = str2num(initialDat{1});
t_f    = str2num(initialDat{2});
u_1    = str2num(initialDat{3});
w_1    = str2num(initialDat{4});
R_1    = str2num(initialDat{5});
s_1    = str2num(initialDat{6});
z_1    = str2num(initialDat{7});
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tspan = [t_1 t_f];
x_0 = [u_1 w_1 R_1 s_1 z_1];
%%%%%%%%%%%%%%% Calculate the basic reproduction number %%%%%%%%%%%%%%%%%%%
R_0 = (xi*e*nu*(alpha*c + b*(beta + lambda)))/(a*(a + c)*(beta + lambda)*(lambda + nu)*(alpha + r + gamma + lambda))

opt = odeset('AbsTol',1e-6,'RelTol',1e-5);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Solve system %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[t,x] = ode45(@SEIRS,tspan,x_0,opt);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Plot results %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
plot(t,x,'linewidth',2);
xlabel('tau');
ylabel('u, w, R, s, z');
legend('Proportion of susceptible humans, u',...
       'Proportion of infectious humans, w',...
       'Proportion of immune humans, R',...
       'Proportion of susceptible vectors, x',...
       'Proportion of infectious vectors, z');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xdot = SEIRS(t,x)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
u = x(1);
w = x(2);
R = x(3);
s = x(4);
z = x(5);
du = lambda*(1 - u) + beta*R + r*w + gamma*w*u - xi*u*z;
dw = nu*(1 - u - R) + gamma.*w^2 - (r + alpha + gamma + lambda + nu)*w;
dR = alpha*w - gamma*w*R - (beta + gamma)*R;     
ds = a*(1 - s) - b*s*w - c*s*R;     
dz = e*(1 - s) - (a + e)*z;
xdot = [du;dw;dR;ds;dz];
end
end