%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% This code solves a simple SIR disease model
%% with no vital dynamics and plots the results.
%% Written by Calistus N. Ngonghala
%% Dept. of Math. West Virginia University, USA
%% email: cngongha@mix.wvu.edu; ngonghala@yahoo.com
%% June 30, 2010
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function SIREpi
clear all;
close all;
clc
set(0,'defaultaxesfontsize',30,'defaulttextinterpreter','latex');
%%%%%%%%%%%%%%%%%%% Parameter Values %%%%%%%%%%%%%%%%%%%%
prompt={'Enter the contact rate, \beta: ',...
        'Enter the removal rate, \nu: ',...
        'Enter the initial time, t_1: ',...
        'Enter the final time, t_f: ',...
        'Enter the initial number of susceptibles individuals, S_1: ',...
        'Enter the initial number of infectious individuals, I_1: ',...
        'Enter the initial number of Recovered individuals, R_1: '};
name='Input for SIR Model with no Vital Dynamics';
numlines=1;
defaultanswer={'0.2','0.1','0','200','999','1','0'};
options.Resize='on';
options.WindowStyle='normal';
options.Interpreter='tex';
parameters=inputdlg(prompt,name,numlines,defaultanswer,options);
%%%%%%%%%%%%%%%%%%%%%% Convert cell arrays to numbers %%%%%%%%%%%%%%%%%%%%%
beta = str2num(parameters{1}); 
nu   = str2num(parameters{2});
t_1  = str2num(parameters{3});
t_f  = str2num(parameters{4});
S_1  = str2num(parameters{5});
I_1  = str2num(parameters{6});
R_1  = str2num(parameters{7});

%%%%%%%%%%%%%%%%% Solve system %%%%%%%%%%%%%%%%%%%%%%%%%
opt = odeset('AbsTol',1e-6,'RelTol',1e-5);
[t x] = ode45(@SIRNoVital,[t_1 t_f],[S_1 I_1 R_1],opt);
R_0 = beta/(nu) % Calculate R_0
%%%%%%%%%%%%%%%%%%%%% Plot Results %%%%%%%%%%%%%%%%%%%%%
figure
plot(t,x(:,1),'g',t,x(:,2),'r',t,x(:,3),'b','LineWidth',2)
title('SIR DISEASE MODEL')
xlabel('time')
ylabel('S, I, R');
legend('S', 'I', 'R');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% Nested Function %%%%%%%%%%%%%%%%%%
%%%%%%%% This function contains the derivatives %%%%%%%%
function xdot = SIRNoVital(t,x)
S = x(1);
I = x(2);
R = x(3);
xdot(1) = - beta*S*I;
xdot(2) = beta*S*I - nu*I;
xdot(3) = nu*I;
xdot = [xdot(1);xdot(2);xdot(3)];
end
end