%% Written by CNN September 29, 2017
   % Code uses current population sizes to determine population sizes at
   % next time step during a simple rubella outbreak
   
clear all % Clear workspace
clc % Clear screen

set(0,'defaultaxesfontsize',30,'defaulttextinterpreter','latex');

beta = 6.8/11; % transmission coefficient for Rubella
gamma = 1/50; % Rubella lassts approximately 11 days

timerange = 0:1:100; % set up the time range

%% Initialization

t(1) = 0;
S(1) = 9999;
I(1) = 1;
R(1) = 0;

%% Loop to determine populations and next time step

for k = 2:length(timerange);
    t(k) = timerange(k);
    S(k) = S(k - 1) - (beta*I(k-1))/(S(k-1) + I(k-1))*S(k-1)*(t(k)-t(k-1));
    I(k) = I(k - 1) + (beta*I(k-1))/(S(k-1) + I(k-1))*S(k-1)*(t(k)-t(k-1)) - gamma*I(k-1)*(t(k)-t(k-1));
    R(k) = R(k-1) + gamma*I(k-1)*(t(k)-t(k-1));
end

 %% Plot results
 
figure('units','normalized','outerposition',[0 0 1 1])

subplot(1,2,1)
plot(t,S,'*b', t,I,'or',t,R,'+g','Markersize', 10)
xlabel('Time in days')
ylabel('Various populations')
Leg_Int_2 = legend('Susceptible, $S$','Infectious, $I$','Recovered, $R$','Interpreter','latex')
set(Leg_Int_2,'Interpreter','latex')

subplot(1,2,2)
plot(t,S,'b', t,I,'r',t,R,'g','LineWidth', 4)
xlabel('Time in days')
ylabel('Various populations')
Leg_Int_2 = legend('Susceptible, $S$','Infectious, $I$','Recovered, $R$')
set(Leg_Int_2,'Interpreter','latex')