%%%%%% Parameter Estimation for ODE Model %%%%%%%%%%%%
% This code estimates 5 parameters for an SIS disease model
% from data.
% Written by Calistus N. N.
% 9 PM 06/20/2012
%% Main function %%

function ModelCalibration1
clear all;
close all;
clc
format long
%%%%%%%% Load data (from excel spreadsheet) %%%%%%%%%
tdata = [0.0,0.46936,0.96002,1.48486,1.99901,2.48272,2.96767,3.95401,4.97244];  
Blooddata = [19.2519,11.3227,6.24831,4.0238,3.22671,3.63359,4.2656,5.15273,5.13606];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% initialize parameter space (a guess is fine) 
parameter(1)=  0.000000000001; 
parameter(2)=  0.000000000001;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Use the "fminsearch" function to carry out minimization
%
%% Initialize states (A guess is fine)
%
x0(1) = 19.2519;
x0(2) = 0;
x0(3) = 0;
x0(4) = 0;
x0(5) = 0;
x0(6) = 0;
x0(7) = 0;
optionsPS = psoptimset('Display','iter','PlotFcns',@psplotbestf);
% Run pattern search solver
[Parameter_min, SS_min] = patternsearch(@SumOfSquaresOfErrors,parameter,[],[],[],[],[],[],optionsPS)
%
% numberOfVariables=2;
% optionsGA = gaoptimset('PlotFcns',@gaplotbestfun);
% % We run GA with the options 'optionsGA' as the tenth argument.
% [Xga,Fga] = ga(@SumOfSquaresOfErrors,numberOfVariables,[],[],[],[],[],[],[],optionsGA)

 disp('Best parameter estimates for: m_{sp}, m_{ps}');
 disp(Parameter_min')
 disp('Minimum value of errors SS:');
 disp(SS_min)

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%This portion of the code defines a function that solves
% the ode model and computes the square of the  differnce 
% between the solution and the data 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
    function SS = SumOfSquaresOfErrors(parameter)
        function xdot = Model(t,x)
            xdot = zeros(7,1);
            xdot(1) = parameter(2)*x(7) - parameter(1)*x(1);
            xdot(2) = parameter(1)*x(1) - parameter(2)*x(2);
            xdot(3) = parameter(2)*x(2) - parameter(2)*x(3);
            xdot(4) = parameter(2)*x(3) - parameter(2)*x(4); 
            xdot(5) = parameter(2)*x(4) - parameter(2)*x(5);
            xdot(6) = parameter(2)*x(5) - parameter(2)*x(6);  
            xdot(7) = parameter(2)*x(6) - parameter(2)*x(7); 
        end
%%
% Prepare to solve ode
tspan = 0:0.001:max(tdata); % set time points
[time,Modelsolution] = ode23s(@Model,tspan,x0); % solve ode
% Plot ode solutions and data
%
set(0,'DefaultAxesFontName', 'Times New Roman','defaultaxesfontsize',20,'defaulttextinterpreter','latex');
%
figure(1)
       % for i = 1:4
         %   subplot(2,2,i)
           % plot(tdata,Blooddata(:,1),'*r','MarkerSize',10);
            plot(tdata,Blooddata,'.r','MarkerSize',25);
            hold on
            plot(time,Modelsolution(:,1),'b','LineWidth',2);
            hold off
            xlabel('time')
            ylabel(['x(' num2str(1) ')']);
        %end
            drawnow

% Identify the predicted values of S and I at the 
% time points in tdata
%
BloodPredicted = interp1(time,Modelsolution,tdata,'spline');
%
% Find the sum of  square of the difference between the solution and the data
%
SS = 0;
        for i = 1:length(tdata)
        %SS = SS + sum((BloodPredicted(i,1)-Blooddata(i,1)).^2);
        SS = SS + sum((BloodPredicted(i,1)-Blooddata(i)).^2);
        end
    end
end