%%%%%% Parameter Estimation for ODE Model %%%%%%%%%%%%
% This code estimates 5 parameters for an SIS disease model
% from data.
% Written by Calistus N. N.
% 9 PM 06/20/2012
%% Main function %%

function ModelCalibrationPig
clear all;
close all;
clc
format long
tdata = [2.46889,4.63652,6.94453,9.14925,11.9144,16.3415,21.4488,26.5464,31.9929,47.8764,64.5844,81.5469,96.6496,127.287,147.11];
BdataSp = [5.19395,3.48797,2.72187,2.4101,2.40414,2.54531,2.23215,1.981,1.58302,1.4111,1.20498,1.22164,1.26546,1.29773,1.16771];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% initialize parameter space (a guess is fine)
parameter(1) = 0.01;
parameter(2) = 0.037737275241981;
parameter(3) = 0.040817217374432;
parameter(4) = 0.605535950238747; 
parameter(5) = 0.037014670660135;
parameter(6) = 0.607142688511364;
parameter(7) = 0.313765421905073; 
parameter(8) = 0.044843125268568;
parameter(9) = 0.096300925084558;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Use the "fminsearch" function to carry out minimization
%
%% Initialize states (A guess is fine)
%
x0(1) = 8.0;
x0(2) = 0;
x0(3) = 0;
x0(4) = 0;
x0(5) = 0;
x0(6) = 0;
x0(7) = 0;
x0(8) = 0;
x0(9) = 0;
x0(10) = 0;
x0(11) = 0;
x0(12) = 0;
x0(13) = 0;
x0(14) = 0;
x0(15) = 0;
x0(14) = 0;
x0(17) = 0;
x0(18) = 0;
x0(19) = 0;
x0(20) = 0;
x0(21) = 0;
x0(22) = 0;
x0(23) = 0;
x0(24) = 0;
x0(25) = 0;
[Parameter_min, SS_min] = fminunc(@SumOfSquaresOfErrors,parameter);
disp('Minimum value of errors SS:');
disp(SS_min)
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%This portion of the code defines a function that solves
% the ode model and computes the square of the  differnce 
% between the solution and the data 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
    function SS = SumOfSquaresOfErrors(parameter)
%
        function xdot = Model(t,x)
            xdot = zeros(25,1);
            xdot(1) = parameter(2)*x(7) + parameter(4)*x(13) + parameter(6)*x(19) + parameter(8)*x(25) - (parameter(1) + parameter(3) + parameter(5) + parameter(7) + parameter(9))*x(1);
            xdot(2) = parameter(1)*x(1) - parameter(2)*x(2);
            xdot(3) = parameter(2)*x(2) - parameter(2)*x(3);
            xdot(4) = parameter(2)*x(3) - parameter(2)*x(4); 
            xdot(5) = parameter(2)*x(4) - parameter(2)*x(5);
            xdot(6) = parameter(2)*x(5) - parameter(2)*x(6);  
            xdot(7) = parameter(2)*x(6) - parameter(2)*x(7);
            xdot(8) = parameter(3)*x(1) - parameter(4)*x(8);
            xdot(9) = parameter(4)*x(8) - parameter(4)*x(9);
            xdot(10) = parameter(4)*x(9) - parameter(4)*x(10); 
            xdot(11) = parameter(4)*x(10) - parameter(4)*x(11);
            xdot(12) = parameter(4)*x(11) - parameter(4)*x(12);  
            xdot(13) = parameter(4)*x(12) - parameter(4)*x(13);
            xdot(14) = parameter(5)*x(1) - parameter(6)*x(14);
            xdot(15) = parameter(6)*x(14) - parameter(6)*x(15);
            xdot(16) = parameter(6)*x(15) - parameter(6)*x(16); 
            xdot(17) = parameter(6)*x(16) - parameter(6)*x(17);
            xdot(18) = parameter(6)*x(17) - parameter(6)*x(18);  
            xdot(19) = parameter(6)*x(18) - parameter(6)*x(19);
            xdot(20) = parameter(7)*x(1) - parameter(8)*x(20);
            xdot(21) = parameter(8)*x(20) - parameter(8)*x(21);
            xdot(22) = parameter(8)*x(21) - parameter(8)*x(22); 
            xdot(23) = parameter(8)*x(22) - parameter(8)*x(23);
            xdot(24) = parameter(8)*x(23) - parameter(8)*x(24);  
            xdot(25) = parameter(8)*x(24) - parameter(8)*x(25);
        end
%%
% Prepare to solve ode
tspan = 0:0.01:max(tdata); % set time points
[time,Modelsolution] = ode23s(@Model,tspan,x0); % solve ode
% Plot ode solutions and data
%
set(0,'DefaultAxesFontName', 'Times New Roman','defaultaxesfontsize',30,'defaulttextinterpreter','latex');
%
figure(1)
semilogy(tdata,BdataSp,'.r','MarkerSize',25);
hold on
semilogy(time,Modelsolution(:,1),'b','LineWidth',2);
hold off
xlabel('time')
ylabel(['x(' num2str(1) ')']);
drawnow

% Identify the predicted values of S and I at the 
% time points in tdata
%
BloodPredicted = interp1(time,Modelsolution,tdata,'spline');
%
% Find the sum of  square of the difference between the solution and the data
%
SS = 0;
        for i = 1:length(tdata)
        %SS = SS + sum((BloodPredicted(i,1)-BdataSp(i,1)).^2);
        SS = SS + sum((BloodPredicted(i,1)-BdataSp(i)).^2);
        end
    end
end