%%%%%% Parameter Estimation for ODE Model %%%%%%%%%%%%
% This code estimates 5 parameters for an SIS disease model
% from data.
% Written by Calistus N. N.
% 9 PM 06/20/2012
%% Main function %%
function SISModelParameterEstimation
%%%%%%%% Load data (from excel spreadsheet) %%%%%%%%%
data = xlsread('Data');
tdata = data(:,1);  
SIdata = data(:,2:3); 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initialize states (A guess is fine)
%
x0(1) = 90; % x0(1) = S0
x0(2) = 10; % x0(2) = I0
%
%% initialize parameter space (a guess is fine)
parameter(1)= 0.05; 
parameter(2)= 0.1;
parameter(3)= 0.01;
parameter(4)= 0.01;
parameter(5)= 0.01;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Use the "fminsearch" function to carry out minimization
%
%options = optimset('Display','iter');
%
[Parameter_min, SS_min] = fminsearch(@SumOfSquaresOfErrors,parameter);
%
disp('Best parameter estimate');
disp(Parameter_min')
disp('Minimum value of errors SS:');
disp(SS_min)
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%This portion of the code defines a function that solves
% the ode model and computes the square of the  differnce 
% between the solution and the data 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
    function SS = SumOfSquaresOfErrors(parameter)
% ODE model equations
% S = x(1) and I = x(2)
%
        function xdot = SIModel(t,x)
            xdot = zeros(2,1);
            xdot(1) = parameter(1)*(x(1) + x(2)) + parameter(2)*x(2) - parameter(3)*x(1)*x(2) - parameter(4)*x(1);
            xdot(2) = parameter(3)*x(1)*x(2) - (parameter(4)+parameter(5)+parameter(2))*x(2);       
        end
% Prepare to solve ode
tspan = 0:1:max(tdata); % set time points
[time,Modelsolution] = ode45(@SIModel,tspan,x0); % solve ode
%% Plot ode solutions and data
figure(1)
        for i = 1:2
            subplot(1,2,i)
            plot(tdata,SIdata(:,i),'*r','MarkerSize',10);
            hold on
            plot(time,Modelsolution(:,i),'b','LineWidth',2);
            hold off
            ylabel(['x(' num2str(i) ')']);
        end
            drawnow

% Identify the predicted values of S and I at the 
% time points in tdata
%
Predicted_SI = interp1(time,Modelsolution,tdata);
%
% Find the sum of  square of the difference between the solution and the data
%
SS = 0;
        for i = 1:length(tdata)
        SS = SS + sum((Predicted_SI(i,:)-SIdata(i,:)).^2);
        end
    end
end