% This function uses a fourth order Runge-Kutta scheme to solve a delayed
% system of differential equations and plot the results. 
% Written by Calistus N. Ngonghala April 1, 2013.
%
    function SS = SumOfSquaresOfErrorsR(p)  
        %
        tdata = [0.0000,0.46936,0.96002,1.48486,1.99901,2.48272,2.96767,3.95401,4.97244];  
        Bdata = [19.2519,11.3227,6.24831,4.0238,3.22671,3.63359,4.2656,5.15273,5.13606];
        N = 1050;  
        B(1) = 19.2519;
        S1(1) = 0;
        S2(1) = 0;
        t(1) = 0;  
        step = 0.01;
        %
        for i = 1:N-1;  
               t(i+1) = t(i) + step;  
            if t(i) < p(3);  
                k11 = - p(1)*B(i);  
                k12 = p(1)*B(i) - p(2)*S1(i);
                k13 = p(2)*S1(i);
                 %
                k21 = - p(1)*(B(i) + 0.5*step*k11);
                k22 = p(1)*(B(i)+ 0.5*step*k12) - p(2)*(S1(i)+ 0.5*step*k12);
                k23 = p(2)*(S1(i) + 0.5*step*k13);  
                %
                k31 = - p(1)*(B(i) + 0.5*step*k21);
                k32 = p(1)*(B(i) + 0.5*step*k22) - p(2)*(S1(i) + 0.5*step*k22);
                k33 = p(2)*(S1(i) + 0.5*step*k23);
                %
                k41 = - p(1)*(B(i) + step*k31);
                k42 = p(1)*(B(i) + step*k32) - p(2)*(S1(i) + 0.5*step*k32);
                k43 = p(2)*(S1(i) + step*k33);
                %
                B(i+1) = B(i) + (k11 + 2*k21 + 2*k31 + k41)*step/6; 
                S1(i+1) = S1(i) + (k12 + 2*k22 + 2*k32 + k42)*step/6;
                S2(i+1) = S2(i) + (k13 + 2*k23 + 2*k33 + k43)*step/6;
            else  
                n = floor((t(i) - p(3) - t(1))/step + 1);
                %
                k11 = p(2)*S1(n) - p(1)*B(i);  
                k12 = p(1)*B(i) - p(2)*S1(i);
                k13 = p(2)*S1(i) - p(2)*S1(n);  
                 %
                k21 = p(2)*S1(n) - p(1)*(B(i) + 0.5*step*k11);
                k22 = p(1)*(B(i) + 0.5*step*k12) - p(2)*(S1(i) + 0.5*step*k12);
                k23 = p(2)*(S1(i) + 0.5*step*k13) - p(2)*S1(n); 
                %
                k31 = p(2)*S1(n) - p(1)*(B(i) + 0.5*step*k21);
                k32 = p(1)*(B(i) + 0.5*step*k22) - p(2)*(S1(i) + 0.5*step*k22);
                k33 = p(2)*(S1(i) + 0.5*step*k23) - p(2)*S1(n);
                %
                k41 = p(2)*S1(n) - p(1)*(B(i) + step*k31);
                k42 = p(1)*(B(i) + step*k32) - p(2)*(S1(i) + 0.5*step*k32);
                k43 = p(2)*(S1(i) + 0.5*step*k33) - p(2)*S1(n);
                %
                 B(i+1) = B(i) + (k11 + 2*k21 + 2*k31 + k41)*step/6; 
                S1(i+1) = S1(i) + (k12 + 2*k22 + 2*k32 + k42)*step/6;
                S2(i+1) = S2(i) + (k13 + 2*k23 + 2*k33 + k43)*step/6;
            end
        end
        %  
        figure(1)
        plot(tdata,Bdata,'.r','MarkerSize',20)
        hold on
        plot(t,B,t,S1,t,S2,'LineWidth',2)
        hold off
        xlabel('Time in hours')
        ylabel('Lymphocytes')
        hh=legend('Data','Blood, $B$','Spleen 1, $S_1$','Spleen 2, $S_2$')
        set(hh,'Interpreter','latex');
        drawnow
        %
        BPredicted = interp1(t,B,tdata,'spline');
        %
        SS = 0;
            for i = 1:length(tdata)
                SS = SS + sum((BPredicted(i)-Bdata(i)).^2);
            end
    end