% This function uses a fourth order Runge-Kutta scheme to solve a delayed
% system of differential equations and plot the results. 
% Written by Calistus N. Ngonghala April 1, 2013.
%
    function SS2 = SumOfSquaresOfErrorsR2(param)  
        %
        tdata2 = [00.0000, 0.472276, 1.23817, 1.74078, 2.26431, 2.76526, 3.24404, 4.26804, 5.26965, 6.24947, 8.2326, 9.25703, 10.2373];  
        Bdata2 = [20.3248, 12.4361,  6.4805,  3.7976,  2.62637, 2.8825,  3.81022, 3.73484, 4.75093, 5.76683, 4.77581,3.94468, 4.12086];
        N = 1050;  
        B(1) = 20.3248;
        S1(1) = 0;
        S2(1) = 0;
        t(1) = 0;  
        step = 0.01;
        %
        for i = 1:N-1;  
               t(i+1) = t(i) + step;  
            if t(i) < param(3);  
                k11 = - param(1)*B(i);  
                k12 = param(1)*B(i) - param(2)*S1(i);
                k13 = param(2)*S1(i);
                 %
                k21 = - param(1)*(B(i) + 0.5*step*k11);
                k22 = param(1)*(B(i)+ 0.5*step*k12) - param(2)*(S1(i)+ 0.5*step*k12);
                k23 = param(2)*(S1(i) + 0.5*step*k13);  
                %
                k31 = - param(1)*(B(i) + 0.5*step*k21);
                k32 = param(1)*(B(i) + 0.5*step*k22) - param(2)*(S1(i) + 0.5*step*k22);
                k33 = param(2)*(S1(i) + 0.5*step*k23);
                %
                k41 = - param(1)*(B(i) + step*k31);
                k42 = param(1)*(B(i) + step*k32) - param(2)*(S1(i) + 0.5*step*k32);
                k43 = param(2)*(S1(i) + step*k33);
                %
                B(i+1) = B(i) + (k11 + 2*k21 + 2*k31 + k41)*step/6; 
                S1(i+1) = S1(i) + (k12 + 2*k22 + 2*k32 + k42)*step/6;
                S2(i+1) = S2(i) + (k13 + 2*k23 + 2*k33 + k43)*step/6;
            else  
                n = floor((t(i) - param(3) - t(1))/step + 1);
                %
                k11 = param(2)*S1(n) - param(1)*B(i);  
                k12 = param(1)*B(i) - param(2)*S1(i);
                k13 = param(2)*S1(i) - param(2)*S1(n);  
                 %
                k21 = param(2)*S1(n) - param(1)*(B(i) + 0.5*step*k11);
                k22 = param(1)*(B(i) + 0.5*step*k12) - param(2)*(S1(i) + 0.5*step*k12);
                k23 = param(2)*(S1(i) + 0.5*step*k13) - param(2)*S1(n); 
                %
                k31 = param(2)*S1(n) - param(1)*(B(i) + 0.5*step*k21);
                k32 = param(1)*(B(i) + 0.5*step*k22) - param(2)*(S1(i) + 0.5*step*k22);
                k33 = param(2)*(S1(i) + 0.5*step*k23) - param(2)*S1(n);
                %
                k41 = param(2)*S1(n) - param(1)*(B(i) + step*k31);
                k42 = param(1)*(B(i) + step*k32) - param(2)*(S1(i) + 0.5*step*k32);
                k43 = param(2)*(S1(i) + 0.5*step*k33) - param(2)*S1(n);
                %
                 B(i+1) = B(i) + (k11 + 2*k21 + 2*k31 + k41)*step/6; 
                S1(i+1) = S1(i) + (k12 + 2*k22 + 2*k32 + k42)*step/6;
                S2(i+1) = S2(i) + (k13 + 2*k23 + 2*k33 + k43)*step/6;
            end
        end
        %  
        figure(2)
        plot(tdata2,Bdata2,'.r','MarkerSize',20)
        hold on
        plot(t,B,t,S1,t,S2,'LineWidth',2)
        hold off
        xlabel('Time in hours')
        ylabel('Lymphocytes')
        hh=legend('Data','Blood, $B$','Spleen 1, $S_1$','Spleen 2, $S_2$')
        set(hh,'Interpreter','latex');
        drawnow
        %
        BPredicted = interp1(t,B,tdata2,'spline');
        %
        SS2 = 0;
            for i = 1:length(tdata2)
                SS2 = SS2 + sum((BPredicted(i)-Bdata2(i)).^2);
            end
    end